%rainbows
% A function which calculates the angular width of a (principle)
% rainbow for all the frequencies of visible light. (Red to Violet).
% The script plots rainbow elevation from an observer as a function of the
% angle of light incident on a raindrop from the normal to the (assumed
% spherical) surface. The derivative of this function is plotted and used
% to illustrate the concept of 'minimum deviation' which focusses light of
% a particular frequency to a certain elevation from the observer - i.e.
% creating the rings of a rainbow. Note the elevation is maximised if the
% deviation is minimised.
%
% LAST UPDATED by Andy French Jan 2018.

function rainbows

%Fontsize for graphs
fsize = 18;

%Number of data points for graphs
N = 100;

%Speed of light in vacuo /ms^-1
c=299792458;

%Angle of elevation of sun (same as shadow angle) /radians
sun_angle = 30*pi/180;

%

%% DETERMINE CONSTANTS FOR SEMI-EMPIRICAL FORMULA FOR REFRACTIVE INDEX N VS FREQUENCY %%

%Refractive index vs frequency /THz of visible light in pure water at 20
%degrees celcius. https://en.wikipedia.org/wiki/Optical_properties_of_water_and_ice
[num,txt,raw] = xlsread('Refractive index of water.xlsx');
f_data = num(:,2).';
n_data = num(:,3).';

%Plot graph, and indicate visible frequency range 405-790THz.
figure('name','refractive index of water','color',[1 1 1]);
plot(f_data, n_data,'k--','linewidth',2 ); hold on;
f_visible = linspace( 405, 790, 100 );
n_visible = interp1( f_data, n_data, f_visible );
for i=1:length(f_visible);
    [R,G,B,colour_str] = colours_from_f( f_visible(i) );
    marker_colour = [R,G,B];
    plot( f_visible(i),n_visible(i),'s','markerfacecolor',[R,G,B],'markeredgecolor',[R,G,B] );
end
ylims = get(gca,'ylim');
plot( [405,405],ylims, 'color',[1,0,0] );
plot( [790,790],ylims, 'color',[127/255,0,1] );
xlabel('Frequency /THz','fontsize',fsize);
ylabel('Refractive index','fontsize',fsize);
title({'Refractive index of water','Visible range is 405-790THz'},'fontsize',fsize);
set(gca,'fontsize',fsize);
grid on;
print( gcf, 'n vs f for water.png','-dpng','-r300' );
close(gcf);

%

%Apply line of best fit to data in visible range
i = find( (f_data < 405 ) | (f_data > 790) );
f_data(i) = [];
n_data(i) = [];
x = (f_data/1000).^2;
y = ( n_data.^2 - 1).^(-2);
[yfit,xfit,r,ybar,xbar,alpha,beta] = bestfitc(x,y);
figure('name','refractive index of water','color',[1 1 1]);
plot( xfit, yfit, 'b-', 'linewidth', 2 ); hold on;
plot( x,y, 'r+', 'linewidth', 2 );
xlabel('x = (f/10^{15}Hz)^2','fontsize',fsize);
ylabel('y = (n^2-1)^{-2}','fontsize',fsize);
title(['y = ',num2str(beta,4),' - ',num2str(-alpha,4),'x'],'fontsize',fsize);
set(gca,'fontsize',fsize);
grid on;
axis tight
print( gcf, 'n vs f for water best fit.png','-dpng','-r300' );
close(gcf);

%Determine model variation of n vs f (in THz)
f = linspace( 405, 790, N );
n = sqrt( ( beta + alpha*( ( f/1000 ).^2 ) ).^(-0.5) + 1 );

%Plot model vs data
figure('name','refractive index of water','color',[1 1 1]);
plot(f,n,'b-',f_data, n_data,'r+','linewidth',2 ); hold on;
xlabel('Frequency /THz','fontsize',fsize);
ylabel('Refractive index','fontsize',fsize);
legend({'Model','Data'},'fontsize',fsize,'location','southeast');
title({'Refractive index of water over visible range 405-790THz',...
    ['(n^2-1)^{-2} = ',num2str(beta,4),' - ',num2str(-alpha,4),'(f/10^{15}Hz)^2']},'fontsize',fsize);
set(gca,'fontsize',fsize);
axis tight
grid on;
print( gcf, 'n vs f for water model vs data.png','-dpng','-r300' );
close(gcf);

%

%% RAINBOW MODEL %%

%Run model for k=1,2
[theta_min2, elev_x1, phi2, theta, ttheta, ff, f, elev1, d_elev_by_d_theta, n, nn] =...
    rainbow_angles( N, 1, alpha, beta, 405, 790 );
[theta_min1, elev_x2, phi1, theta, ttheta, ff, f, elev2, d_elev_by_d_theta, n, nn] =...
    rainbow_angles( N, 2, alpha, beta, 405, 790 );

%Plot surface graph of elevation as a function of theta and frequency /THz
%(This is for k=1)
figure('color',[1 1 1],'numbertitle','off','name','elevation_surface');
surf( (180/pi)*ttheta,ff,(180/pi)*elev1,(180/pi)*elev1 )
shading interp
view(2)
interp_colormap(N);
colorbar('fontsize',fsize)
axis tight
set(gca,'fontsize',fsize)
xlabel('\theta /deg','fontsize',fsize)
ylabel('Frequency /THz','fontsize',fsize)
title('Elevation of deflected beam /deg','fontsize',fsize)
print(gcf,'elevation_surface.png','-dpng','-r300')
close(gcf);

%Plot the same information as a graph, with elev(theta) colour coded by
%visible colour
figure('color',[1 1 1],'name','elev vs theta')
axes('nextplot','add');
for i=1:N
    %Determine colours
    [R,G,B,colour_str] = colours_from_f( f(i) );
    
    %Plot horizontal lines indicating the extremum angle i.e. the
    %elevation of the raninbow
    plot( theta*180/pi, (180/pi)*elev_x1(i)*ones(1,N),'-',...
        'linewidth',0.5, 'color', [R,G,B] );
    plot( theta*180/pi, (180/pi)*elev_x2(i)*ones(1,N),'-',...
        'linewidth',0.5,'color', [R,G,B] );
    
    %Plot elevation angle vs incident angle theta
    plot( theta*180/pi, (180/pi)*elev1(i,:), 'color', [R,G,B] );
    plot( theta*180/pi, (180/pi)*elev2(i,:), 'color', [R,G,B] );
end
set(gca,'fontsize',fsize)
grid on
axis tight
box on;
xlabel('\theta /deg','fontsize',fsize)
ylabel('\epsilon /deg','fontsize',fsize)
title({'Elevation of deflected beam /deg',...
    ['Primary \epsilon=',num2str(min(elev_x1)*180/pi,3),'^o to ,'...
    num2str(max(elev_x1)*180/pi,3),'^o, Secondary \epsilon=',...
    num2str(min(elev_x2*180/pi),3),'^o to ',...
    num2str(max(elev_x2)*180/pi,3),'^o']},'fontsize',fsize)
print(gcf,'elevation vs theta.png','-dpng','-r300')
close(gcf);

%Plot graphs of the extremum value of elevation and theta as a function of frequency.
figure('color',[1 1 1],'numbertitle','off','name','rainbow_elevation');
hold on
for i=1:N
    [R,G,B,colour_str] = colours_from_f( f(i) );
    plot(f(i),(180/pi)*elev_x1(i),'.','markersize',18,'color',[R,G,B]);
    plot(f(i),(180/pi)*elev_x2(i),'.','markersize',18,'color',[R,G,B]);
end
set(gca,'fontsize',fsize)
grid on
axis tight
ylim([40,54])
box on;
xlabel('Frequency /THz','fontsize',fsize)
ylabel('\epsilon /deg','fontsize',fsize)
title('Elevation of single and double rainbows','fontsize',fsize)
print(gcf,'elevation vs f.png','-dpng','-r300')
close(gcf);

%Plot graphs of the extremum value of refracted angle phi as a function of
%frequency, and also the critical angle asin(1/n)
figure('color',[1 1 1],'numbertitle','off','name','rainbow_phi');
phi_crit = (180/pi)*asin(1./n);
plot( f, phi_crit, 'k-','linewidth',2 );
hold on
for i=1:N
    [R,G,B,colour_str] = colours_from_f( f(i) );
    plot(f(i),(180/pi)*phi1(i),'.','markersize',18,'color',[R,G,B]);
    plot(f(i),(180/pi)*phi2(i),'.','markersize',18,'color',[R,G,B]);
end
set(gca,'fontsize',fsize)
grid on
axis tight
box on;
xlabel('Frequency /THz','fontsize',fsize)
ylabel('\phi /deg','fontsize',fsize)
title('Refraction angle of single and double rainbows','fontsize',fsize)
print(gcf,'phi vs f.png','-dpng','-r300')
close(gcf);

%Draw rainbow arcs as they would be vsisible from an observer on the ground
figure('color',[1 1 1],'numbertitle','off','name','rainbow_elevation');
hold on;
r1 = sin(elev_x1)*cos(sun_angle);
r2 = sin(elev_x2)*cos(sun_angle);
beta1 = acos( 1 - sin(elev_x1-sun_angle)./r1 );
beta2 = acos( 1 - sin(elev_x2-sun_angle)./r2 );
y0 = r1(end)*cos(beta1(end));
for i=1:N
    %Plot primary rainbow
    [R,G,B,colour_str] = colours_from_f( f(i) );
    b1 = linspace( -beta1(i),beta1(i),N);
    y1 = r1(i)*cos(b1) - y0;
    x1 = r1(i)*sin(b1);
    plot( x1,y1,'-','color',[R,G,B]);
    
    %Plot secondary rainbow
    [R,G,B,colour_str] = colours_from_f( f(i) );
    b2 = linspace( -beta2(i),beta2(i),N);
    y2 = r2(i)*cos(b2) - y0;
    x2 = r2(i)*sin(b2);
    plot( x2,y2,'-','color',[R,G,B]);
end
axis equal;
axis tight;
ylims = get(gca,'ylim');
ylim([0,ylims(2)]);
axis off
title(['Solar angle \alpha=',num2str(sun_angle*180/pi),'^o'],'fontsize',fsize);
set(gca,'fontsize',fsize);
print(gcf,['visible rainbow arc solar angle=',...
    strrep( num2str(sun_angle*180/pi),'.','p' ),'.png'],'-dpng','-r300');
close(gcf);

%%

%Line of best fit function yfit = m*x +c, with product moment correlation
%coefficient r
function [yfit,xfit,r,ybar,xbar,m,c] = bestfitc(x,y)

%Find any x or y values that are NaN or Inf
ignore = isnan(abs(x)) | isnan(abs(y)) | isinf(abs(x)) | isinf(abs(y));
x(ignore) = [];
y(ignore) = [];

%Compute line of best fit
xbar = mean(x);
ybar = mean(y);
xybar = mean(x.*y);
Vx = mean(x.^2 ) - xbar^2;
Vy = mean(y.^2 ) - ybar^2;
COVxy = mean(x.*y) - xbar*ybar;
m = COVxy/Vx;
c = ybar - m*xbar;
r = COVxy/sqrt( Vx*Vy );
yfit = m*x + c;
xfit = x;

%%

%colours_from_f
% Function which provides the R,G,B values( within interval [0,1] )
% of visible light depending on the frequency /THz
function [R,G,B,colour_str] = colours_from_f(f)
if f < 405
    R = NaN; G = NaN; B = NaN; colour_str = 'Infra Red';
elseif (f>=405) && ( f < 480 )
    R = 1; G = 0; B = 0; colour_str = 'Red';
elseif (f>=480) && ( f < 510 )
    R = 1; G = 127/255; B = 0; colour_str = 'Orange';
elseif (f>=510) && ( f < 530 )
    R = 1; G = 1; B = 0; colour_str = 'Yellow';
elseif (f>=530) && ( f < 600 )
    R = 0; G = 1; B = 0; colour_str = 'Green';
elseif (f>=600) && ( f < 620 )
    R = 0; G = 1; B = 1; colour_str = 'Cyan';
elseif (f>=620) && ( f < 680 )
    R = 0; G = 0; B = 1; colour_str = 'Blue';
elseif (f>=680) && ( f <= 790 )
    R = 127/255; G = 0; B = 1; colour_str = 'Violet';
else
    R = NaN; G = NaN; B = NaN; colour_str = 'Ultra Violet';
end

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%%

%rainbow_angles
% Descartes model of rainbow angles
function [theta_min, elev_min, phi, theta, ttheta, ff, f, elev, d_elev_by_d_theta n, nn] =...
    rainbow_angles( N, k, alpha, beta, fmin, fmax )

%Determine model variation of n vs f (in THz)
f = linspace( fmin, fmax, N );
n = sqrt( ( beta + alpha*( ( f/1000 ).^2 ) ).^(-0.5) + 1 );

%Compute angles that minimise elev for a given value of n
%and corresponding minimum value of elevation. These are the angles
%of the rainbow from the observer.
theta_min = asin( sqrt( ( (k+1)^2 - n.^2 )/( (k+1)^2 -1 ) ) );
elev_min = (2*k+2)*asin( sin( theta_min )./n )  - 2*theta_min - pi*(k-1);

%Define angle of internal reflection phi
phi = asin( sin(theta_min)./n );

% Define array of angles between (parrallel) incident sunlight and the
% surface normal of the spherical raindrop. Note range is from 0 to 90
% degrees since the ray geometry asserts that the incident %ray is 'above'
% the observer. This construct is valid since the solution is rotationally
% symmetric %about the direction of sunlight. i.e. the elevation angle
% describes the angular width of the 'cone of observation' whose base is
% the rainbow.
theta=linspace(0,pi/2,N);
[ttheta,ff] = meshgrid(theta,f);
nn = sqrt( ( beta + alpha*( ( ff/1000 ).^2 ) ).^(-0.5) + 1 );

%Compute elevation angles of deviated rays as a function of theta and n
%and the first derivative of elevation angle w.r.t. theta
elev = (2*k+2)*asin( sin( ttheta )./nn )  - 2*ttheta - pi*(k-1);
d_elev_by_d_theta = (2*k+2)*cos(ttheta)./sqrt( (nn.^2) - (sin(ttheta).^2) ) - 2;

%Ensure positive root is plotted
if elev_min(1)<0
    elev_min = -elev_min;
    elev = -elev;
end

%End of code.



